@@ -6,6 +6,7 @@ import itertools
 import logging
 import re
 from collections import defaultdict, namedtuple
+from json import JSONDecodeError
 
 import six
 from django.core.exceptions import ImproperlyConfigured
@@ -990,13 +991,15 @@ class SQLCompiler(BaseSQLCompiler):
 
             pk, params = self.build_params_and_pk()
             url = get_resource_path(self.query.model, pk)
-            response = self.connection.cursor().get(
-                url,
-                params=params
-            )
-            self.raise_on_response(url, params, response)
+            response = self.make_request(params, url)
+
+            try:
+                json = response.json()
+            except JSONDecodeError:
+                extra = {'params': params, 'response': response}
+                logger.error('json decode error while calling {}; retrying'.format(url), extra=extra)
+                json = self.make_request(params, url).json()
 
-            json = response.json()
             meta = self.get_meta(json, response)
             if meta:
                 # pagination and others thing
@@ -1038,6 +1041,14 @@ class SQLCompiler(BaseSQLCompiler):
         result = self.result_iter(response_reader)
         return result
 
+    def make_request(self, params, url):
+        response = self.connection.cursor().get(
+            url,
+            params=params
+        )
+        self.raise_on_response(url, params, response)
+        return response
+
 
 class SQLInsertCompiler(SQLCompiler):
     def resolve_data_n_files(self, obj):
@@ -1058,7 +1069,7 @@ class SQLInsertCompiler(SQLCompiler):
                 file = fieldfile.name
 
                 if file is not None:  # field value can be None....
-                    files[field.column] = (file.name, file, file.content_type)
+                    files[field.column] = (file.name, file, getattr(file, 'content_type', None))
                 else:
                     data[field.column] = None
             else:
@@ -1213,6 +1224,7 @@ class FakeCursor(object):
     """
     a fake cursor with rowcount
     """
+
     def __init__(self, rowcount):
         self.rowcount = rowcount
 
@@ -1307,8 +1319,7 @@ class SQLUpdateCompiler(SQLCompiler):
                     # str => we don't change it since it's not comming from our custom storage
                     pass
                 else:
-                    files[field.column] = (file.name, file, file.content_type)
-
+                    files[field.column] = (file.name, file, getattr(file, 'content_type', None))
             else:
                 fieldname = field.concrete and field.db_column or field.name
                 data[fieldname] = field.get_db_prep_save(val, connection=self.connection)
@@ -1336,7 +1347,7 @@ class SQLUpdateCompiler(SQLCompiler):
                     if result.status_code not in (200, 202, 204):
                         raise FakeDatabaseDbAPI2.ProgrammingError(
                             "error while updating %s.pk=%s with data=%s,files=%s.\n%s" % (
-                             query.model.__name__, id_, data, files, message_from_response(result))
+                                query.model.__name__, id_, data, files, message_from_response(result))
                         )
 
                     # update object instance using result data if possible
